/*******************************************************************************
 * Copyright (c) 2024-2025 R3944Realms. All rights reserved.
 * <p>
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * <p>
 *     http://www.apache.org/licenses/LICENSE-2.0
 * <p>
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/

package com.r3944realms.dg_lab.websocket.handler.client;

import com.r3944realms.dg_lab.api.operation.ClientOperation;
import com.r3944realms.dg_lab.api.websocket.message.data.PowerBoxData;

/**
 * The type Default client operation.
 */
public class DefaultClientOperation implements ClientOperation {
    @Override
    public void ClientStartingHandler() {
        //NOOP
    }

    @Override
    public void ClientStartedHandler() {
        //NOOP
    }

    @Override
    public void ClientStartingErrorHandler() {
        //NOOP
    }

    @Override
    public void ClientStoppingHandler() {
        //NOOP
    }

    @Override
    public void ClientStoppingErrorHandler() {
        //NOOP
    }

    @Override
    public void ClientStoppedHandler() {
        //NOOP
    }

    @Override
    public void QrCodeUrlHandler(String qrCodeUrl) {
        //NOOP
        System.out.println(qrCodeUrl);
    }

    @Override
    public void ShowQrCodeHandler() {
        //NOOP
    }

    @Override
    public void ConnectSuccessfulNoticeHandler() {
        //NOOP
    }

    @Override
    public void DisconnectHandler(final PowerBoxData data) {
        //NOOP
    }

    @Override
    public void ErrorHandler(PowerBoxData data) {
        //NOOP
    }

    @Override
    public void HeartBeatHandler(PowerBoxData data) {
        //NOOP
    }

    @Override
    public void OtherMessageHandler(PowerBoxData data) {
        //NOOP
    }
}
